/*******************************************************************************
* Copyright (C) 2014 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/

/******************************************************************************/
/** \file i2clcd.c
 **
 ** This file contains the definitions and functions required to initialize the 
 ** I2C bus and interfacing the LCD.
 **
 ** History:
 **   - 2014-09-18  1.0  Deepak Naik  First version.
 **
 ******************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "i2clcd.h"
#include "stdio.h"
#include "string.h"

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/
/*LCD I2C Slave address*/
#define LCD_SLAVE_ADDRESS (0x7Cu)

/* LCD Address */
#define LCD_CMD_ADDR                   (0x00u)
#define LCD_DDRAM_1ST_ADDR             (0x80u)
#define LCD_DDRAM_2ND_ADDR             (0xC0u)

/* Control Data */
#define LCD_INIT_FUN_SET1              (0x38u)
#define LCD_INIT_FUN_SET2              (0x39u)
#define LCD_INIT_OSC_FREQ              (0x14u)
#define LCD_INIT_CONTRAST_SET          (0x75u)
#define LCD_INIT_PW_ICON               (0x5Fu)
#define LCD_INIT_FOLLOWER              (0x6Bu)
#define LCD_INIT_DISP_ON               (0x0Cu)
#define LCD_INIT_DISP_ON_CURSOR        (0x0Du)
#define LCD_INIT_ENTRY_MODE_SET        (0x06u)
#define LCD_CLEAR_DISP                 (0x01u)

#define LCD_INIT_DISP_OFF              (0x08u)
#define LCD_RETURN_DISP                (0x02u)
#define LCD_SCROLL_LEFT                (0x18u)

#define LCD_CURSOR_LEFT                (0x10u)
#define LCD_CURSOR_RIGHT               (0x14u)

#define LCD_DEV_ADDR                   (0x3Eu)

/// I2C Baudrate in KBit/s
#define LCD_I2C_BAUDRATE               (400000ul)



/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/
static en_result_t I2cLcdCmdTrans      (uint8_t u8Cmd);

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/

/**
 ******************************************************************************
 ** \brief    Send an array of 8-bit data
 **
 ** \param    [in] pu8Data Pointer to array of bytes in mem
 ** \param    [in] u32Size size of the array to be sent
 ******************************************************************************/
static void I2cSendBytes(uint8_t* pu8Data, uint32_t u32Size)
{
    uint32_t u32I;
    for(u32I = 0u; u32I < u32Size; u32I++)
    {
        I2cLcd_I2cSendByte(pu8Data[u32I]);
    }
}

/**
 ******************************************************************************
 ** \brief Initialization of P23 for LCD reset signal
 ******************************************************************************/
void I2cLcd_InitReset(void)
{
    bFM4_GPIO_ADE_AN30 = 0u;
    bFM4_GPIO_PDOR2_P3 = 1u;
    bFM4_GPIO_DDR2_P3 = 1u;
    bFM4_GPIO_PFR2_P3 = 0u;
}

/**
 ******************************************************************************
 ** \brief Initialize MFS4-BT for background illumination
 ******************************************************************************/
void I2cLcd_InitBG(void)
{
    bFM4_GPIO_PFR2_P4 = 1u;
    bFM4_GPIO_ADE_AN29 = 0u;
    FM4_GPIO->EPFR13 &= ~(0x3ul << 10ul);
    FM4_GPIO->EPFR13 |=  (0x2ul << 10ul);
    FM4_BT13_PWM->TMCR = 0x0810u;
    FM4_BT13_PWM->TMCR2 = 0x0u;
    FM4_BT13_PWM->STC = 0x20u;
    FM4_BT13_PWM->PCSR = 255u;
    FM4_BT13_PWM->PDUT = 0u;
    FM4_BT13_PWM->TMCR |= 0x02u;
    FM4_BT13_PWM->TMCR |= 0x01u;
}

/**
 ******************************************************************************
 ** \brief Initialize MFS4 for I2C and BT for background illumination
 **
 ** \retval Ok             Initialization process finished
 ******************************************************************************/
en_result_t I2cLcd_Init(void)
{
  volatile uint32_t u32Delay;

  I2cLcd_InitReset();                           // LCD reset signal

  bFM4_GPIO_PDOR2_P3 = 0u;                      // I2CLCD SETRESET;
  u32Delay = 50000ul;
  while (--u32Delay);

  bFM4_GPIO_PDOR2_P3 = 1u;                      // I2CLCD CLEARRESET;
  u32Delay = 50000ul;
  while (--u32Delay);

  // LCD initialization sequence
  I2cLcdCmdTrans (LCD_INIT_FUN_SET1);           // function set
  I2cLcdCmdTrans (LCD_INIT_FUN_SET2);           // function set
  I2cLcdCmdTrans (LCD_INIT_OSC_FREQ);           // internal osc
  I2cLcdCmdTrans (LCD_INIT_CONTRAST_SET);       // contrast set
  I2cLcdCmdTrans (LCD_INIT_PW_ICON);            // pw/icon/contrast
  I2cLcdCmdTrans (LCD_INIT_FOLLOWER);           // follower control
  I2cLcdCmdTrans (LCD_INIT_DISP_ON);            // Display ON
  I2cLcdCmdTrans (LCD_CLEAR_DISP);              // clear display
  I2cLcdCmdTrans (LCD_INIT_ENTRY_MODE_SET);     // entry mode set
  I2cLcdCmdTrans (LCD_RETURN_DISP);             // init position

  I2cLcd_SetBG(0u);
  u32Delay = 50000ul;
  while (--u32Delay);                           // guard time between STOP and START for display

  I2cLcd_InitBG();

  return Ok;
}


/**
 ******************************************************************************
 ** \brief Change LCD background light brightness
 **
 ** \param [in]  u8LcdBrightness Brightness of LCD backlight, 255: full, 0: dark
 **
 ** \retval Ok             LED value configured
 ******************************************************************************/
en_result_t I2cLcd_SetBG(uint16_t u16LcdBrightness)
{
  FM4_BT13_PWM->PDUT = u16LcdBrightness;

  return Ok;
}


/**
 ******************************************************************************
 ** \brief Send command to LCD module
 **
 ** \param  [in] u8Cmd Command
 **
 ** \retval Ok   LCD command transmitted
 ******************************************************************************/
static en_result_t I2cLcdCmdTrans(uint8_t u8Cmd)
{
  uint8_t  au8Data[2u];
  volatile uint32_t u32Delay = 250ul;

  // transfer command to LCD
  au8Data[0u] = LCD_CMD_ADDR;
  au8Data[1u] = u8Cmd;

  // Write data to LCD (generate start condition)
  I2cLcd_I2cStart(LCD_SLAVE_ADDRESS | I2C_RW_WRITE);

  // Write data to LCD module (data write)
  I2cSendBytes((uint8_t*) au8Data, 2u);

  // Write data to LCD module (generate stop condition)
  I2cLcd_I2cReadStop();

  while (--u32Delay)                            // Guard time between STOP and START for display
  {}
      
  return Ok;
}


/**
 ******************************************************************************
 ** \brief Erases all text on LCD module
 **
 ** \retval Ok             LCD cleared
 ******************************************************************************/
en_result_t I2cLcd_Clear(void)
{
  return I2cLcdCmdTrans (LCD_CLEAR_DISP);
}

/**
 ******************************************************************************
 ** \brief Sets Cursor on LCD to HOME position
 **
 ** \retval Ok             Cursor set to position 0
 ******************************************************************************/
en_result_t I2cLcd_ReturnHome(void)
{
  return I2cLcdCmdTrans (LCD_RETURN_DISP);
}


/**
 ******************************************************************************
 ** \brief Write a text on LCD module
 **
 ** \param [in]  u8Line      1: upper LCD line, 2: lower line
 ** \param [in]  pcText      aray of 16 characters containing the text to be
 **                         displayed
 **
 ** \retval Ok                       Text written
 **         ErrorInvalidParameter    If provided parameter u8Line is not 1 or 2
 ******************************************************************************/
en_result_t I2cLcd_Write(uint8_t u8Line, char * pcText)
{
  en_result_t enErrorCode = Ok;
  uint8_t     au8Cmd[1u];
  volatile uint32_t u32Delay = 200ul;
  uint8_t u8MaxLen = strlen(pcText);

  if ((u8MaxLen > 16u) && (1u == u8Line))
  {
     I2cLcd_Write(2u, pcText + 16u);
  }
  if (u8MaxLen > 16u)
  {
      u8MaxLen = 16u;
  }
  if (1u == u8Line)
  {
    I2cLcdCmdTrans(LCD_DDRAM_1ST_ADDR);
  }
  else if (2u == u8Line)
  {
    I2cLcdCmdTrans(LCD_DDRAM_2ND_ADDR);
  }
  else
  {
    return ErrorInvalidParameter;
  }

  // Write data to LCD (generate start condition)
  I2cLcd_I2cStart(LCD_SLAVE_ADDRESS | I2C_RW_WRITE);
  au8Cmd [0u] = 0x40u;                                       // Memory address
  I2cSendBytes((uint8_t*) au8Cmd, 1u);

  // Write data to LCD module (data write)
  I2cSendBytes((uint8_t*) pcText, u8MaxLen);

  // Write data to LCD module (generate stop condition)
  I2cLcd_I2cWriteStop();
  while (--u32Delay);                                       // Guard time between STOP and START for display
  return enErrorCode;
}


/*****************************************************************************
 ** \brief    Send I2C Byte
 **
 ** \param    [in] u8Data Data
 **
 ** \return   int32_t 0 = 0k / -1 = Error
 *****************************************************************************/

int32_t I2cLcd_I2cSendByte(uint8_t u8Data)
{
  volatile uint32_t u32Timeout;

  FM4_MFS14->TDR = u8Data;                                  // Send data to TSC
  FM4_MFS14->IBCR = 0xB0u;                                  // WSEL = 1, ACKE = 1, Clear INT flag

  u32Timeout = I2C_TIME_OUT;
  while(!(FM4_MFS14->IBCR & 0x01u))                         // Wait for transmission complete
  {                                                         // via INT flag
      u32Timeout--;
      if (0u == u32Timeout)
      {
          return -1;
      }
  }

  if (FM4_MFS14->IBCR & 0x02u)                              // BER == 1? ->  error
  {
    return -1;
  }

  if (!(FM4_MFS14->IBCR & 0x40u))                           // ACT == 0? ->  error
  {
    return -1;
  }

  // MSS is set, no reserved address

  if (FM4_MFS14->IBSR & 0x40u)                              // RACK == 1? ->  error
  {
    return -1;
  }

  if (!(FM4_MFS14->IBSR & 0x10u))                           // TRX == 0? ->  error
  {
    return -1;
  }

  return 0u;
}


/*****************************************************************************
 ** \brief   Start of I2C Communication with 1st Data Byte
 **
 ** \param   [in] u8Data Byte
 **
 ** \return  0 = 0k / -1 = Error
 *****************************************************************************/
int32_t I2cLcd_I2cStart(uint8_t u8Data)
{
  uint32_t u16ErrorCount = 0u;
  volatile uint32_t u32Timeout;

  FM4_MFS14->TDR  = (u8Data);                               // Send data to TSC
  FM4_MFS14->IBCR = 0x80u;                                  // MSS = 1: Master Mode

  do
  {
    u32Timeout = I2C_TIME_OUT;
    while(!(FM4_MFS14->IBCR & 0x01u))                       // Wait for transmission complete
    {                                                       // via INT flag
        u32Timeout--;
        if (0u == u32Timeout)
        {
            return -1;
        }
    }
    if (FM4_MFS14->IBCR & 0x02u)                            // BER == 1? ->  error
    {
      return -1;
    }

    if (!(FM4_MFS14->IBCR & 0x40u))                         // ACT == 0? ->  error
    {
      return -1;
    }

    // MSS is set ...

    if (FM4_MFS14->IBSR & 0x40u)                            // RACK == 1? ->  busy or error
    {
      FM4_MFS14->IBCR = 0xE0u;                              // MMS = CSS = ACKE = 1: Try restart (if busy)
      u16ErrorCount++;

      if (u16ErrorCount > I2C_TIME_OUT)
      {
        return -1;                                          // Too much retrials
      }
    }
    else
    {
      u16ErrorCount = 0u;
    }

  }while (u16ErrorCount);

  if (!(FM4_MFS14->IBSR & 0x10u))                           // TRX == 0? ->  error
  {
     return -1;
  }

  return 0u;
}


/*****************************************************************************
 ** \brief    Stop I2C Write Communication
 **
 ** \return   int32_t 0 = Ok / -1 = Timed out error
 *****************************************************************************/
int32_t I2cLcd_I2cWriteStop(void)
{
  volatile uint32_t u32Timeout;

  u32Timeout = I2C_STOP_TIMEOUT;

  FM4_MFS14->IBCR = 0x20u;                                  // MMS = CSS = INT = 0, ACKE = 1

  while(1u == (FM4_MFS14->IBSR & 0x01u))
  {
    u32Timeout--;
    if (0u == u32Timeout)
    {
      return -1;
    }
  }
  return 0;
}


/*****************************************************************************
 ** \brief   Stop I2C READ Communication
 **
 ** \return  int32_t 0 Ok
 *****************************************************************************/
int32_t I2cLcd_I2cReadStop(void)
{
  FM4_MFS14->IBCR = 0x01u;                                  // INT = 1, MMS = ACKE = 0
  return 0u;
}



/*****************************************************************************
 ** \brief    Initialize I2C Bus
 **
 ** \return   boolean_t TRUE when I2C bus is initialized
 *****************************************************************************/
boolean_t I2c_Init(void)
{
   I2cBusClear(&FM4_GPIO->DDR6,
               &FM4_GPIO->PFR6,
               &FM4_GPIO->PDOR6,
               &FM4_GPIO->PDIR6,
               (1ul << 0xDul),
               (1ul << 0xCul));

   bFM4_GPIO_PFR6_PC = 1u;                                  // Pin 200 (SOT14_1)
   bFM4_GPIO_PFR6_PD = 1u;                                  // Pin 199 (SCK14_1)

   FM4_GPIO->EPFR17 &= ~(0x3ul << 16ul);                    // SIN14S
   FM4_GPIO->EPFR17 &= ~(0x3ul << 18ul);                    // SOT14B
   FM4_GPIO->EPFR17 &= ~(0x3ul << 20ul);                    // SCK14B

   FM4_GPIO->EPFR17 |= (0x2ul << 18ul);                     // SOT14B
   FM4_GPIO->EPFR17 |= (0x2ul << 20ul);                     // SCK14B


   FM4_MFS14->BGR  = (__HCLK / 2ul) / 400000ul - 1ul;       // BGR = PCLK / BAUDRATE - 1
   FM4_MFS14->SCR |= 0x80u;                                 // Reset (UPCL = 1)
   FM4_MFS14->SMR  = 0x80u;                                 // operation mode 4, no interrupts, no test
                                                            // mode, no wake up

   FM4_MFS14->ISBA = 0x00u;

   FM4_MFS14->NFCR |= 0x0008u;                              // Enable noise filter for I2C ch. 1 for 60-80MHz PCLK operation

   FM4_MFS14->ISMK = 0x80u;                                 // Enable I2C interface

   return TRUE;
}


/*****************************************************************************
 ** \brief    The following function checks if a slave device holds the SDA
 **           line low before initializing I2C. This can happen if a read
 **           operation from a slave was interrupted, eg. due to MCU reset.
 **           In case some slave device is holding the bus low, up to nine
 **           clock pulses are generated until the line is released.
 **
 ** \param    [in] pu32DDR pointer to DDR register of I/O port to which SCL & SDA are configured
 ** \param    [in] pu32PFR pointer to PFR register of I/O port to which SCL & SDA are configured
 ** \param    [in] pu32PDOR pointer to PDOR register of I/O port to which SCL & SDA are configured
 ** \param    [in] pu32PDIR pointer to PDIR register of I/O port to which SCL & SDA are configured
 ** \param    [in] u32SclMask Mask bit corresponding to the SCL line
 ** \param    [in] u32SdaMask Mask bit corresponding to the SDA line
 *****************************************************************************/
void I2cBusClear(volatile uint32_t* pu32DDR, volatile uint32_t* pu32PFR, volatile uint32_t* pu32PDOR, volatile uint32_t* pu32PDIR, uint32_t u32SclMask, uint32_t u32SdaMask)
{
  volatile uint32_t u32Dly;
  uint8_t u8I;

  *pu32DDR  &= ~(u32SclMask | u32SdaMask);                   // input (to read bus state)
  *pu32PDOR &= ~(u32SclMask | u32SdaMask);                  // PF low
  *pu32PFR  &= ~(u32SclMask | u32SdaMask);                   // Disable MFS peripheral output

  if (0u == (*pu32PDIR & u32SdaMask))                       // If someone holds SDA low, generate some dummy clock pulses until line is released
  {
    *pu32DDR |= u32SclMask;                                 // set SCL to output
    for (u8I = 0u; u8I < 9u; u8I++)
    {
      u32Dly = 0xFFFFul;
      while (--u32Dly)
      {}
      *pu32PDOR |= u32SclMask;                              // PF2 --> high (SCL)
      
      u32Dly = 0xFFFFul;
      while (--u32Dly)
      {}
          
      *pu32PDOR &= ~u32SclMask;                             // PF2 --> low (SCL)
      if (*pu32PDIR & u32SdaMask)
      {
          break;                                            // stop if SDA was released meanwhile
      }
    }
  }
  u32Dly = 0xFFFFul;
  while (--u32Dly)
  {}
      
  *pu32PDOR |= (u32SclMask | u32SdaMask);                   // SDA + SCL high
  *pu32DDR  |= (u32SclMask | u32SdaMask);
  
  u32Dly = 0xFFFFul;
  while (--u32Dly)
  {}

  // MFS I2C has to be re-initialized after this!
}

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
